@@ -1,28 +1,16 @@
-/***************************************************************************
- *   Copyright 2006-2007 Alexander Dymo  <adymo@kdevelop.org>       *
- *                                                                         *
- *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU Library General Public License as       *
- *   published by the Free Software Foundation; either version 2 of the    *
- *   License, or (at your option) any later version.                       *
- *                                                                         *
- *   This program is distributed in the hope that it will be useful,       *
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
- *   GNU General Public License for more details.                          *
- *                                                                         *
- *   You should have received a copy of the GNU Library General Public     *
- *   License along with this program; if not, write to the                 *
- *   Free Software Foundation, Inc.,                                       *
- *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.         *
- ***************************************************************************/
+/*
+    SPDX-FileCopyrightText: 2006-2007 Alexander Dymo <adymo@kdevelop.org>
+
+    SPDX-License-Identifier: LGPL-2.0-or-later
+*/
+
 #include "controller.h"
 
 #include <QMap>
 #include <QList>
 #include <QEvent>
 #include <QMouseEvent>
-#include <QApplication>
+#include <QCoreApplication>
 
 #include <KSharedConfig>
 
@@ -105,7 +93,7 @@ Controller::Controller(QObject *parent)
 void Controller::init()
 {
     loadSettings();
-    qApp->installEventFilter(this);
+    QCoreApplication::instance()->installEventFilter(this);
 }
 
 Controller::~Controller()
@@ -433,11 +421,6 @@ void Controller::notifyViewAdded(Sublime::AreaIndex*, Sublime::View *view)
     emit viewAdded(view);
 }
 
-void Controller::setStatusIcon(Document * document, const QIcon & icon)
-{
-    document->setStatusIcon(icon);
-}
-
 void Controller::loadSettings()
 {
     Q_D(Controller);
