@@ -7,7 +7,9 @@
 
 #include "context.h"
 
+#include <QHash>
 #include <QRegularExpression>
+#include <QSet>
 #include <QStandardPaths>
 
 #include <interfaces/icore.h>
@@ -264,7 +266,7 @@ public:
             if (functionType->indexedArgumentsSize() && didAddParentheses) {
                 view->setCursorPosition(word.start() + KTextEditor::Cursor(0, repl.size() - 1));
             }
-            auto returnTypeIntegral = functionType->returnType().cast<IntegralType>();
+            auto returnTypeIntegral = functionType->returnType().dynamicCast<IntegralType>();
             if ( restEmpty && !funcptr && returnTypeIntegral && returnTypeIntegral->dataType() == IntegralType::TypeVoid ) {
                 // function returns void and rest of line is empty -- nothing can be done with the result
                 if (functionType->indexedArgumentsSize() ) {
@@ -507,15 +509,33 @@ QString& elideStringRight(QString& str, int length)
     return str;
 }
 
+constexpr int maxBestMatchCompletionPriority = CCP_SuperCompletion;
+
 /**
- * @return Value suited for @ref CodeCompletionModel::MatchQuality in the range [0.0, 10.0] (the higher the better)
+ * @return Value suited for @ref CodeCompletionModel::MatchQuality in the range [1, 10] (the higher the better)
  *
  * See https://clang.llvm.org/doxygen/CodeCompleteConsumer_8h_source.html for list of priorities
- * They (currently) have a range from [-3, 80] (the lower, the better)
+ * They (currently) are in the range [0, 80] (the lower, the better). Nevertheless, we are only setting priority
+ * until maxBestMatchCompletionPriority (20), so we better build the value around it.
  */
-int codeCompletionPriorityToMatchQuality(unsigned int completionPriority)
+int matchQualityFromBestMatchCompletionPriority(int completionPriority)
 {
-    return 10u - qBound(0u, completionPriority, 80u) / 8;
+    Q_ASSERT(completionPriority >= 0);
+    Q_ASSERT(completionPriority <= maxBestMatchCompletionPriority);
+
+    constexpr int maxMatchQuality = 10;
+    auto matchQuality = maxMatchQuality - maxMatchQuality * completionPriority / maxBestMatchCompletionPriority;
+    Q_ASSERT(matchQuality >= 0);
+
+    // KTextEditor considers a completion with matchQuality == 0 not suitable.
+    // DeclarationItem::data() considers matchQuality == 0 invalid and does not return it.
+    // Avoid zero special case by increasing matchQuality 0 to 1.
+    constexpr int minSuitableMatchQuality = 1;
+    matchQuality = std::max(minSuitableMatchQuality, matchQuality);
+
+    Q_ASSERT(matchQuality >= minSuitableMatchQuality);
+    Q_ASSERT(matchQuality <= maxMatchQuality);
+    return matchQuality;
 }
 
 int adjustPriorityForType(const AbstractType::Ptr& type, int completionPriority)
@@ -532,11 +552,11 @@ int adjustPriorityForType(const AbstractType::Ptr& type, int completionPriority)
         } else if (whichType == AbstractType::TypeDelayed) {
             completionPriority += modifier;
         } else if (whichType == AbstractType::TypeAlias) {
-            auto aliasedType = type.cast<TypeAliasType>();
-            return adjustPriorityForType(aliasedType ? aliasedType->type() : AbstractType::Ptr(), completionPriority);
+            auto aliasedType = type.staticCast<TypeAliasType>();
+            return adjustPriorityForType(aliasedType->type(), completionPriority);
         } else if (whichType == AbstractType::TypeFunction) {
-            auto functionType = type.cast<FunctionType>();
-            return adjustPriorityForType(functionType ? functionType->returnType() : AbstractType::Ptr(), completionPriority);
+            auto functionType = type.staticCast<FunctionType>();
+            return adjustPriorityForType(functionType->returnType(), completionPriority);
         }
     } else {
         completionPriority += modifier;
@@ -546,7 +566,7 @@ int adjustPriorityForType(const AbstractType::Ptr& type, int completionPriority)
 }
 
 /// Adjusts priority for the @p decl
-int adjustPriorityForDeclaration(Declaration* decl, unsigned int completionPriority)
+int adjustPriorityForDeclaration(Declaration* decl, int completionPriority)
 {
     if(completionPriority < CCP_LocalDeclarationSimiliar || completionPriority > CCP_SuperCompletion){
         return completionPriority;
@@ -717,26 +737,35 @@ public:
 
     /// Add type for matching. This type'll be used for filtering look-ahead items
     /// Only items with @p type will be returned through @sa matchedItems
-    void addMatchedType(const IndexedType& type)
+    /// @param matchQuality @p type's match quality
+    void addMatchedType(const IndexedType& type, int matchQuality)
     {
-        if (type.isValid()) {
-            matchedTypes.insert(type);
+        Q_ASSERT_X(matchQuality > 0, Q_FUNC_INFO, "The completion must be suitable.");
+        if (!type.isValid()) {
+            return;
         }
+        auto& lookAheadMatchQuality = matchedTypeToMatchQuality[type];
+        // Use the highest among match qualities associated with the type, because currently
+        // supported look-ahead completions are as useful as CCP_LocalDeclaration, and the
+        // corresponding match quality is unlikely to be lower than @p matchQuality.
+        // The following statement correctly assigns @p matchQuality if @p type was just inserted into
+        // matchedTypeToMatchQuality, because then lookAheadMatchQuality == int{} == 0 < matchQuality.
+        lookAheadMatchQuality = std::max(lookAheadMatchQuality, matchQuality);
     }
 
     /// @return look-ahead items that math given types. @sa addMatchedType
-    QList<CompletionTreeItemPointer> matchedItems()
+    QList<CompletionTreeItemPointer> matchedItems() const
     {
         QList<CompletionTreeItemPointer> lookAheadItems;
         for (const auto& pair: qAsConst(possibleLookAheadDeclarations)) {
             auto decl = pair.first;
-            if (matchedTypes.contains(decl->indexedType())) {
+            if (const auto matchQuality = matchedTypeToMatchQuality.value(decl->indexedType())) {
                 auto parent = pair.second;
                 const QLatin1String access = (parent->abstractType()->whichType() == AbstractType::TypePointer)
                                  ? QLatin1String("->") : QLatin1String(".");
                 const QString text = parent->identifier().toString() + access + decl->identifier().toString();
                 auto item = new DeclarationItem(decl, text, {}, text);
-                item->setMatchQuality(8);
+                item->setMatchQuality(matchQuality);
                 lookAheadItems.append(CompletionTreeItemPointer(item));
             }
         }
@@ -776,7 +805,7 @@ private:
                     }
 
                     if (localDecl->abstractType()->whichType() == AbstractType::TypeIntegral) {
-                        if (auto integralType = declaration->abstractType().cast<IntegralType>()) {
+                        if (auto integralType = declaration->abstractType().dynamicCast<IntegralType>()) {
                             if (integralType->dataType() == IntegralType::TypeVoid) {
                                 continue;
                             }
@@ -792,8 +821,8 @@ private:
     // Declaration and it's context
     using DeclarationContext = QPair<Declaration*, Declaration*>;
 
-    /// Types of declarations that look-ahead completion items can have
-    QSet<IndexedType> matchedTypes;
+    /// Types of declarations that look-ahead completion items can have and their match qualities
+    QHash<IndexedType, int> matchedTypeToMatchQuality;
 
     // List of declarations that can be added to the Look Ahead group
     // Second declaration represents context
@@ -950,12 +979,27 @@ ClangCodeCompletionContext::ClangCodeCompletionContext(const DUContextPointer& c
                                                  allUnsaved.data(), allUnsaved.size(),
                                                  clang_defaultCodeCompleteOptions()));
 
-            if (m_results && m_results->NumResults) {
-                QMetaObject::invokeMethod(&s_memberAccessReplacer, "replaceCurrentAccess", Qt::QueuedConnection,
-                                          Q_ARG(MemberAccessReplacer::Type, MemberAccessReplacer::DotToArrow));
+            m_valid = false;
+
+            if (!m_results || m_results->NumResults == 0) {
+                return;
             }
 
-            m_valid = false;
+            const auto diagnostic = clang_codeCompleteGetDiagnostic(m_results.get(), 0);
+            const ClangString str(clang_getDiagnosticCategoryText(diagnostic));
+
+            // This is unfortunately not documented anywhere in clang
+            const bool isParseIssue = (qstrcmp(str.c_str(), "Parse Issue") == 0);
+            clang_disposeDiagnostic(diagnostic);
+
+            // Do not perform subsitution if we get a parsing issue with the ->
+            if (isParseIssue) {
+                return;
+            }
+
+            QMetaObject::invokeMethod(&s_memberAccessReplacer, "replaceCurrentAccess", Qt::QueuedConnection,
+                                      Q_ARG(MemberAccessReplacer::Type, MemberAccessReplacer::DotToArrow));
+
             return;
         }
     }
@@ -1086,14 +1130,14 @@ QList<CompletionTreeItemPointer> ClangCodeCompletionContext::completionItems(boo
         FunctionSignatureState signatureState = Before;
         //END function signature parsing
 
-        std::function<void (CXCompletionString)> processChunks = [&] (CXCompletionString completionString) {
+        const auto processChunks = [&](CXCompletionString completionString, const auto& self) -> void {
             const uint chunks = clang_getNumCompletionChunks(completionString);
             for (uint j = 0; j < chunks; ++j) {
                 const auto kind = clang_getCompletionChunkKind(completionString, j);
                 if (kind == CXCompletionChunk_Optional) {
                     completionString = clang_getCompletionChunkCompletionString(completionString, j);
                     if (completionString) {
-                        processChunks(completionString);
+                        self(completionString, self);
                     }
                     continue;
                 }
@@ -1158,7 +1202,7 @@ QList<CompletionTreeItemPointer> ClangCodeCompletionContext::completionItems(boo
             }
         };
 
-        processChunks(result.CompletionString);
+        processChunks(result.CompletionString, processChunks);
 
         // we have our own implementation of an override helper
         // TODO: use the clang-provided one, if available
@@ -1228,20 +1272,21 @@ QList<CompletionTreeItemPointer> ClangCodeCompletionContext::completionItems(boo
                     declarationItem = new DeclarationItem(found, typed, resultType, replacement);
                 }
 
-                const unsigned int completionPriority = adjustPriorityForDeclaration(found, clang_getCompletionPriority(result.CompletionString));
-                const bool bestMatch = completionPriority <= CCP_SuperCompletion;
+                const auto completionPriority =
+                    adjustPriorityForDeclaration(found, clang_getCompletionPriority(result.CompletionString));
+                const bool bestMatch = completionPriority <= maxBestMatchCompletionPriority;
 
                 //don't set best match property for internal identifiers, also prefer declarations from current file
                 const auto isInternal = found->indexedIdentifier().identifier().toString().startsWith(QLatin1String("__"));
                 if (bestMatch && !isInternal) {
-                    const int matchQuality = codeCompletionPriorityToMatchQuality(completionPriority);
+                    const auto matchQuality = matchQualityFromBestMatchCompletionPriority(completionPriority);
                     declarationItem->setMatchQuality(matchQuality);
 
                     // TODO: LibClang missing API to determine expected code completion type.
                     if (auto functionType = found->type<FunctionType>()) {
-                        lookAheadMatcher.addMatchedType(IndexedType(functionType->returnType()));
+                        lookAheadMatcher.addMatchedType(IndexedType(functionType->returnType()), matchQuality);
                     }
-                    lookAheadMatcher.addMatchedType(found->indexedType());
+                    lookAheadMatcher.addMatchedType(found->indexedType(), matchQuality);
                 } else {
                     declarationItem->setInheritanceDepth(completionPriority);
 
