@@ -27,7 +27,6 @@
 #include <tests/testcore.h>
 #include <shell/shellextension.h>
 
-#include <kcoreaddons_version.h>
 #include <KIO/Global>
 #include <KProcess>
 #include <KSharedConfig>
@@ -853,11 +852,7 @@ void GdbTest::testAttach()
     QTest::qWait(100);
 
     auto *session = new TestDebugSession;
-#if KCOREADDONS_VERSION < QT_VERSION_CHECK(5, 78, 0)
-    session->attachToProcess(debugeeProcess.pid());
-#else
     session->attachToProcess(debugeeProcess.processId());
-#endif
     WAIT_FOR_STATE(session, DebugSession::PausedState);
 
     breakpoints()->addCodeBreakpoint(QUrl::fromLocalFile(fileName), 39); // } after foo();
@@ -890,11 +885,7 @@ void GdbTest::testManualAttach()
                                                          QStringLiteral("unittests/gdb_script_empty"))));
     QVERIFY(session->startDebugging(&cfg, m_iface));
 
-#if KCOREADDONS_VERSION < QT_VERSION_CHECK(5, 78, 0)
-    session->addCommand(MI::NonMI, QStringLiteral("attach %0").arg(debugeeProcess.pid()));
-#else
     session->addCommand(MI::NonMI, QStringLiteral("attach %0").arg(debugeeProcess.processId()));
-#endif
     WAIT_FOR_STATE(session, DebugSession::PausedState);
 
     session->run();
