@@ -1,20 +1,8 @@
-/* This file is part of the KDE libraries
-   Copyright (C) 2007 David Nolden <david.nolden.kdevelop@art-master.de>
+/*
+    SPDX-FileCopyrightText: 2007 David Nolden <david.nolden.kdevelop@art-master.de>
 
-   This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
-   License version 2 as published by the Free Software Foundation.
-
-   This library is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
-
-   You should have received a copy of the GNU Library General Public License
-   along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301, USA.
- */
+    SPDX-License-Identifier: LGPL-2.0-only
+*/
 
 #ifndef PROJECT_FILE_QUICKOPEN
 #define PROJECT_FILE_QUICKOPEN
@@ -25,6 +13,9 @@
 
 #include <util/path.h>
 
+#include <utility>
+#include <vector>
+
 namespace KDevelop {
 class IProject;
 class ProjectFileItem;
@@ -35,8 +26,9 @@ class ProjectFileItem;
  */
 struct ProjectFile
 {
-    ProjectFile()
-    {}
+    ProjectFile() = default;
+    explicit ProjectFile(const KDevelop::ProjectFileItem*);
+
     KDevelop::Path path;
     // project root folder url
     KDevelop::Path projectPath;
@@ -48,12 +40,34 @@ struct ProjectFile
     bool outsideOfProject = false;
 };
 
+inline void swap(ProjectFile& a, ProjectFile& b) noexcept
+{
+    using std::swap;
+    swap(a.path, b.path);
+    swap(a.projectPath, b.projectPath);
+    swap(a.indexedPath, b.indexedPath);
+    swap(a.outsideOfProject, b.outsideOfProject);
+}
+
 inline bool operator<(const ProjectFile& left, const ProjectFile& right)
 {
     if (left.outsideOfProject != right.outsideOfProject) {
+        // place the less interesting generated files at the end
         return !left.outsideOfProject;
     }
-    return left.path < right.path;
+    const int comparison = left.path.compare(right.path, Qt::CaseInsensitive);
+    if (comparison != 0) {
+        return comparison < 0;
+    }
+    // Only paths that are completely, case-sensitively equal are considered
+    // duplicates. Comparing indexed paths here when the paths are
+    // case-insensitively equal ensures that:
+    // * the duplicates are adjacent and thus detected by std::unique;
+    // * binary search algorithms find only case-sensitively equal elements.
+    // OpenFilesDataProvider default-initializes all its indexed paths making
+    // them all equal to each other. This is fine because OpenFilesDataProvider
+    // doesn't use std::unique or binary search algorithms.
+    return left.indexedPath < right.indexedPath;
 }
 
 Q_DECLARE_TYPEINFO(ProjectFile, Q_MOVABLE_TYPE);
@@ -135,7 +149,7 @@ private:
     // project files sorted by their url
     // this is done so we can limit ourselves to a relatively fast
     // filtering without any expensive sorting in reset().
-    QVector<ProjectFile> m_projectFiles;
+    std::vector<ProjectFile> m_projectFiles;
 };
 
 /**
