@@ -34,6 +34,7 @@
 #include <util/expandablelineedit.h>
 
 #include "outputmodel.h"
+#include "outputwidgetconfig.h"
 #include "toolviewdata.h"
 #include <debug.h>
 
@@ -45,7 +46,7 @@ QString validFilterInputToolTip()
 }
 
 OutputWidget::OutputWidget(QWidget* parent, const ToolViewData* tvdata)
-    : QWidget( parent )
+    : QWidget(parent)
     , m_tabwidget(nullptr)
     , m_stackwidget(nullptr)
     , data(tvdata)
@@ -57,6 +58,7 @@ OutputWidget::OutputWidget(QWidget* parent, const ToolViewData* tvdata)
     , m_focusOnSelect(nullptr)
     , m_filterInput(nullptr)
     , m_filterAction(nullptr)
+    , m_outputWidgetConfig(nullptr)
 {
     setWindowTitle(i18nc("@title:window", "Output View"));
     setWindowIcon(tvdata->icon);
@@ -168,6 +170,35 @@ OutputWidget::OutputWidget(QWidget* parent, const ToolViewData* tvdata)
         }
     }
 
+    if (!data->configSubgroupName.isEmpty()
+        && (data->type & KDevelop::IOutputView::MultipleView || data->type & KDevelop::IOutputView::HistoryView)) {
+        m_outputWidgetConfig = new OutputWidgetConfig(data->configSubgroupName, data->title, this);
+        connect(m_outputWidgetConfig, &OutputWidgetConfig::settingsChanged, this, [this]() {
+            const auto maxViewCount = m_outputWidgetConfig->maxViewCount();
+            // don't close views when view limit is not enabled
+            if (!maxViewCount.has_value()) {
+                return;
+            }
+            if (data->type & KDevelop::IOutputView::MultipleView) {
+                closeFirstViewsWhileTooMany(*m_tabwidget, *maxViewCount);
+            } else {
+                closeFirstViewsWhileTooMany(*m_stackwidget, *maxViewCount);
+            }
+        });
+
+        QAction* const openConfigAction = KStandardAction::preferences(
+            m_outputWidgetConfig,
+            [this]() {
+                m_outputWidgetConfig->openDialog(this);
+            },
+            this);
+        openConfigAction->setText(
+            i18nc("@action %1: output type, e.g. Build or Run", "Configure %1 Output", data->title));
+        openConfigAction->setShortcut(QKeySequence());
+        openConfigAction->setShortcutContext(Qt::WidgetWithChildrenShortcut);
+        insertAction(separator, openConfigAction);
+    }
+
     addActions(data->actionList);
 
     connect( data, &ToolViewData::outputAdded,
@@ -292,27 +323,61 @@ void OutputWidget::removeOutput( int id )
     enableActions();
 }
 
-void OutputWidget::closeActiveView()
+bool OutputWidget::closeView(const QWidget* view)
 {
-    QWidget* widget = m_tabwidget->currentWidget();
-    if( !widget )
+    Q_ASSERT(view);
+    Q_ASSERT(qobject_cast<const QAbstractItemView*>(view));
+
+    const auto fvIt = constFindFilteredView(static_cast<const QAbstractItemView*>(view));
+    if (fvIt == m_views.cend()) {
+        return false;
+    }
+
+    const auto id = fvIt.key();
+    if (!(data->outputdata.value(id)->behaviour & KDevelop::IOutputView::AllowUserClose)) {
+        return false;
+    }
+
+    data->plugin->removeOutput(id);
+    enableActions();
+    return true;
+}
+
+template<class ViewContainer>
+void OutputWidget::closeFirstViewIfTooMany(const ViewContainer& viewContainer)
+{
+    if (!m_outputWidgetConfig) {
         return;
-    const auto ids = m_views.keys();
-    for (int id : ids) {
-        if (m_views.value(id).view == widget)
-        {
-            OutputData* od = data->outputdata.value(id);
-            if( od->behaviour & KDevelop::IOutputView::AllowUserClose )
-            {
-                data->plugin->removeOutput( id );
-            }
+    }
+    const auto maxViewCount = m_outputWidgetConfig->maxViewCount();
+    Q_ASSERT(!maxViewCount.has_value() || *maxViewCount > 0);
+    if (maxViewCount.has_value() && viewContainer.count() > *maxViewCount) {
+        closeView(viewContainer.widget(0));
+    }
+}
+
+template<class ViewContainer>
+void OutputWidget::closeFirstViewsWhileTooMany(const ViewContainer& viewContainer, int maxViewCount)
+{
+    Q_ASSERT(maxViewCount > 0);
+    while (viewContainer.count() > maxViewCount) {
+        if (!closeView(viewContainer.widget(0))) {
+            break; // Closing a view usually succeeds. Prevent endless loop if it fails.
         }
     }
-    enableActions();
+}
+
+void OutputWidget::closeActiveView()
+{
+    Q_ASSERT(m_tabwidget);
+    if (const auto* view = m_tabwidget->currentWidget()) {
+        closeView(view);
+    }
 }
 
 void OutputWidget::closeOtherViews()
 {
+    Q_ASSERT(m_tabwidget);
     QWidget* widget = m_tabwidget->currentWidget();
     if (!widget)
         return;
@@ -385,8 +450,8 @@ void OutputWidget::activateIndex(const QModelIndex &index, QAbstractItemView *vi
         return;
     QModelIndex sourceIndex = index;
     QModelIndex viewIndex = index;
-    auto fvIt = findFilteredView(view);
-    if (fvIt != m_views.end() && fvIt->proxyModel) {
+    const auto fvIt = constFindFilteredView(view);
+    if (fvIt != m_views.cend() && fvIt->proxyModel) {
         auto proxy = fvIt->proxyModel;
         if ( index.model() == proxy ) {
             // index is from the proxy, map it to the source
@@ -434,8 +499,8 @@ void OutputWidget::selectItem(SelectionMode selectionMode)
     eventuallyDoFocus();
 
     auto index = view->currentIndex();
-    auto fvIt = findFilteredView(view);
-    if (fvIt != m_views.end() && fvIt->proxyModel) {
+    const auto fvIt = constFindFilteredView(view);
+    if (fvIt != m_views.cend() && fvIt->proxyModel) {
         auto proxy = fvIt->proxyModel;
         if ( index.model() == proxy ) {
             // index is from the proxy, map it to the source
@@ -508,10 +573,12 @@ QTreeView* OutputWidget::createListView(int id)
             if( data->type & KDevelop::IOutputView::MultipleView )
             {
                 m_tabwidget->addTab(listview, data->outputdata.value(id)->title);
+                closeFirstViewIfTooMany(*m_tabwidget);
             } else
             {
                 const int index = m_stackwidget->addWidget(listview);
                 m_stackwidget->setCurrentIndex(index);
+                closeFirstViewIfTooMany(*m_stackwidget);
             }
         } else
         {
@@ -521,7 +588,7 @@ QTreeView* OutputWidget::createListView(int id)
                 layout()->addWidget(listview);
             } else
             {
-                listview = m_views.begin().value().view;
+                listview = m_views.cbegin().value().view;
                 newView = false;
             }
         }
@@ -669,16 +736,16 @@ void OutputWidget::outputFilter(const QString& filter)
     }
     proxyModel->setFilterRegularExpression(regex);
 
-    updateFilterInputAppearance(fvIt);
+    updateFilterInputAppearance(constIterator(fvIt));
 }
 
 void OutputWidget::updateFilter(int index)
 {
     QWidget *view = (data->type & KDevelop::IOutputView::MultipleView)
         ? m_tabwidget->widget(index) : m_stackwidget->widget(index);
-    auto fvIt = findFilteredView(qobject_cast<QAbstractItemView*>(view));
+    const auto fvIt = constFindFilteredView(qobject_cast<QAbstractItemView*>(view));
 
-    const QString filterText = fvIt == m_views.end() ? QString{} : fvIt->filter.pattern();
+    const QString filterText = fvIt == m_views.cend() ? QString{} : fvIt->filter.pattern();
     if (filterText.isEmpty()) {
         m_filterInput->clear();
     } else {
@@ -699,19 +766,32 @@ void OutputWidget::setTitle(int outputId, const QString& title)
     }
 }
 
-OutputWidget::FilteredViewIterator OutputWidget::findFilteredView(QAbstractItemView* view)
+auto OutputWidget::constIterator(FilteredViews::iterator it) -> FilteredViews::const_iterator
 {
-    for (auto it = m_views.begin(); it != m_views.end(); ++it) {
-        if (it->view == view) {
-            return it;
-        }
-    }
-    return m_views.end();
+    return static_cast<FilteredViews::const_iterator>(it);
 }
 
-void OutputWidget::updateFilterInputAppearance(FilteredViewIterator currentView)
+template<typename ForwardIt>
+ForwardIt OutputWidget::findFilteredView(ForwardIt first, ForwardIt last, const QAbstractItemView* view)
 {
-    if (currentView == m_views.end() || currentView->filter.isValid()) {
+    return std::find_if(first, last, [view](const FilteredView& filteredView) {
+        return filteredView.view == view;
+    });
+}
+
+auto OutputWidget::findFilteredView(const QAbstractItemView* view) -> FilteredViews::iterator
+{
+    return findFilteredView(m_views.begin(), m_views.end(), view);
+}
+
+auto OutputWidget::constFindFilteredView(const QAbstractItemView* view) const -> FilteredViews::const_iterator
+{
+    return findFilteredView(m_views.cbegin(), m_views.cend(), view);
+}
+
+void OutputWidget::updateFilterInputAppearance(FilteredViews::const_iterator currentView)
+{
+    if (currentView == m_views.cend() || currentView->filter.isValid()) {
         m_filterInput->setPalette(QPalette{});
         m_filterInput->setToolTip(validFilterInputToolTip());
     } else {
@@ -725,3 +805,5 @@ void OutputWidget::updateFilterInputAppearance(FilteredViewIterator currentView)
                   currentView->filter.errorString()));
     }
 }
+
+#include "moc_outputwidget.cpp"
